@@ -33,6 +33,7 @@
 
 #include "file-data.h"
 #include "file-utils.h"
+#include "glib-utils.h"
 #include "fr-command.h"
 #include "fr-command-cfile.h"
 
@@ -115,10 +116,10 @@ list__process_line (char     *line,
 	fdata = file_data_new ();
 
 	fields = split_line (line, 2);
-	fdata->size = atol (fields[1]);
+	fdata->size = g_ascii_strtoull (fields[1], NULL, 10);
 	g_strfreev (fields);
 
-	if (fdata->size == -1)
+	if (fdata->size == 0)
 		fdata->size = get_file_size (comm->filename);
 
 	filename = get_uncompressed_name_from_archive (comm, comm->filename);
@@ -551,5 +552,16 @@ fr_command_cfile_new (FRProcess         *process,
 	fr_command_construct (comm, process, filename);
 	FR_COMMAND_CFILE (comm)->compress_prog = prog;
 
+	if (prog == FR_COMPRESS_PROGRAM_GZIP)
+		comm->file_type = FR_FILE_TYPE_GZIP;
+	else if (prog == FR_COMPRESS_PROGRAM_BZIP)
+		comm->file_type = FR_FILE_TYPE_BZIP;
+	else if (prog == FR_COMPRESS_PROGRAM_BZIP2)
+		comm->file_type = FR_FILE_TYPE_BZIP2;
+	else if (prog == FR_COMPRESS_PROGRAM_COMPRESS)
+		comm->file_type = FR_FILE_TYPE_COMPRESS;
+	else if (prog == FR_COMPRESS_PROGRAM_LZOP)
+		comm->file_type = FR_FILE_TYPE_LZOP;
+
 	return comm;
 }
